# -*- encoding: utf-8 -*-

import logging

from django.contrib.auth.backends import BaseBackend

from .models import DiscordUser

log = logging.getLogger(__name__)


class DiscordAuthenticationBackend(BaseBackend):
    """
    Authentication Backend for Discord Users.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def authenticate(self, request, discord_user_data: dict) -> DiscordUser:
        """
        Creates or finds an existing user, and returns it.
        "Authenticate" may not be the best name for this method...
        """

        log.debug(discord_user_data)

        existing_user = self.get_user(discord_user_data["id"])
        log.debug("authenticating, does user exist: %s", bool(existing_user))
        return existing_user or DiscordUser.objects.create_user(discord_user_data)


    def get_user(self, user_id: int) -> DiscordUser | None:
        """
        Fetch a user from their ID.
        Returns NoneType if not found.
        """

        try:
            return DiscordUser.objects.get(pk=user_id)
        except DiscordUser.DoesNotExist:
            return None
